// -*- C++ -*-
// ACL:license
// ----------------------------------------------------------------------
// This software and ancillary information (herein called "SOFTWARE")
// called POOMA (Parallel Object-Oriented Methods and Applications) is
// made available under the terms described here.  The SOFTWARE has been
// approved for release with associated LA-CC Number LA-CC-98-65.
// 
// Unless otherwise indicated, this SOFTWARE has been authored by an
// employee or employees of the University of California, operator of the
// Los Alamos National Laboratory under Contract No. W-7405-ENG-36 with
// the U.S. Department of Energy.  The U.S. Government has rights to use,
// reproduce, and distribute this SOFTWARE. The public may copy, distribute,
// prepare derivative works and publicly display this SOFTWARE without 
// charge, provided that this Notice and any statement of authorship are 
// reproduced on all copies.  Neither the Government nor the University 
// makes any warranty, express or implied, or assumes any liability or 
// responsibility for the use of this SOFTWARE.
// 
// If SOFTWARE is modified to produce derivative works, such modified
// SOFTWARE should be clearly marked, so as not to confuse it with the
// version available from LANL.
// 
// For more information about POOMA, send e-mail to pooma@acl.lanl.gov,
// or visit the POOMA web page at http://www.acl.lanl.gov/pooma/.
// ----------------------------------------------------------------------
// ACL:license

//-----------------------------------------------------------------------------
// Functions: 
//   setExternalGuards
//-----------------------------------------------------------------------------

#ifndef POOMA_FIELD_FIELDFUNCTIONS_H
#define POOMA_FIELD_FIELDFUNCTIONS_H

//-----------------------------------------------------------------------------
// Overview: 
// 
// setExternalGuards(f,val) sets the external guards to have the value val.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Includes:
//-----------------------------------------------------------------------------

#include "Field/Field.h"
#include "BConds/ConstantFaceBC.h"

//-----------------------------------------------------------------------------
// Full Description:
//
// setExternalGuards(f,val) sets the external guards to have the value val.
// This is implemented using the ConstantFaceBC boundary condition. We
// simply create one on the fly, invoke it, and then delete it.
//-----------------------------------------------------------------------------

template <class Geom, class T, class EngineTag>
void setExternalGuards(const Field<Geom,T,EngineTag> &f, const T &val)
{
  for (int i = 0; i < 2*Geom::dimensions; ++i)
    {
      BCondItem *b = ConstantFaceBC<T>(i,val).create(f);
      b->applyBoundaryCondition();
      delete b;
    }
}

#endif // POOMA_FIELD_FIELDFUNCTIONS_H

// ACL:rcsinfo
// ----------------------------------------------------------------------
// $RCSfile: FieldFunctions.h,v $   $Author: swhaney $
// $Revision: 1.5 $   $Date: 2000/03/07 13:17:08 $
// ----------------------------------------------------------------------
// ACL:rcsinfo
